
template<u8 regRef16, u8 regRef8, bool write, bool dis> void Core_z80::op_ld_1() {
	if (dis) displacement = readPc(3);
	if (regRef8 == _DATA) data8 = readPc(3);
	if (regRef16 == _DATA) {
		data16 = readPc(3);
		data16 |= readPc(3) << 8;
	}

	if (dis) sync(regRef8 == _DATA ? 2 : 5);

	if (write) writeByte(
		(regRef16 == _DATA ? data16 : *regPTR16[regRef16]) + (dis ? i8(displacement) : 0),
		regRef8 == _DATA ? data8 : *regPTR[regRef8], 3);
	else *regPTR[regRef8] = readByte(
		(regRef16 == _DATA ? data16 : *regPTR16[regRef16]) + (dis ? i8(displacement) : 0), 3);
}

template<u8 regRef16, bool write> void Core_z80::op_ld_2() {
	data16 = readPc(3);
	data16 |= readPc(3) << 8;
	if (write) {
		writeByte(data16, *regPTR16[regRef16] & 0xFF, 3);
		writeByte(data16 + 1, (*regPTR16[regRef16] >> 8) & 0xFF, 3);
	} else {
		*regPTR16[regRef16] = readByte(data16, 3);
		*regPTR16[regRef16] |= readByte(data16 + 1, 3) << 8;
	}
}

template<u8 regRef8_1, u8 regRef8_2> void Core_z80::op_ld_3() {
	if (regRef8_1 == _IXH || regRef8_1 == _IYH || regRef8_1 == _IXL || regRef8_1 == _IYL
	|| regRef8_2 == _IXH || regRef8_2 == _IYH || regRef8_2 == _IXL || regRef8_2 == _IYL) {
		if (regRef8_2 == _DATA) data8 = readPc(1);
		else sync(1);
	}
	else if (regRef8_2 == _DATA) data8 = readPc(3);

	*regPTR[regRef8_1] = regRef8_2 == _DATA ? data8 : *regPTR[regRef8_2];
	if (regRef8_2 == _I || regRef8_2 == _R) {
		reg.bit.s = *regPTR[regRef8_2] & 0x80;
		reg.bit.z = *regPTR[regRef8_2] == 0;
		reg.bit.h = 0;
		reg.bit.pv = intr.iff2;
		reg.bit.n = 0;
		setUndocumentedFlags(*regPTR[regRef8_1]);
	}
	if (regRef8_2 == _I || regRef8_2 == _R || regRef8_1 == _I || regRef8_1 == _R) sync(1);
}

template<bool ix, u8 regRef8, u8 mode, u8 bit> void Core_z80::op_ld_4() {
	data8 = readByte((ix ? reg.w.IX : reg.w.IY) + i8(displacement), 4);
	switch ( mode ) {
		case _SET: data8 = doSet(bit, data8); break;
		case _RESET: data8 = doRes(bit, data8); break;
		case _RL: data8 = doRL<false, true> (data8); break;
		case _RLC: data8 = doRL<true, true> (data8); break;
		case _RR: data8 = doRR<false, true> (data8); break;
		case _RRC: data8 = doRR<true, true> (data8); break;
		case _SLL: data8 = doSL<false> (data8); break;
		case _SLA: data8 = doSL<true> (data8); break;
		case _SRL: data8 = doSR<false> (data8); break;
		case _SRA: data8 = doSR<true> (data8); break;
	}
	if (regRef8 != _DATA) *regPTR[regRef8] = data8;
	writeByte((ix ? reg.w.IX : reg.w.IY) + i8(displacement), data8, 3);
}

template<u16 regRef16_1, u16 regRef16_2> void Core_z80::op_ld_5() {
	if (regRef16_2 == _DATA) {
		data16 = readPc(3);
		data16 |= readPc(3) << 8;
	} else sync(2);
	*regPTR16[regRef16_1] = regRef16_2 == _DATA ? data16 : *regPTR16[regRef16_2];
}

template<bool ei> void Core_z80::op_xI() {
	intr.iff1 = intr.iff2 = ei;
	intr.delay = true;
}

template<u8 regRef> void Core_z80::op_pop() {
	*regPTR16[regRef] = readStack();
}

template<u8 regRef> void Core_z80::op_push() {
	sync(1);
	writeStack(*regPTR16[regRef]);
}

template<bool inc, bool repeat> void Core_z80::op_cpx() {
	data8 = readByte(reg.w.HL, 3);
	bool temp = reg.bit.c;
	data8 = doSub8(reg.b.A, data8);
	data8 -= reg.bit.h;
	reg.bit.unused_bit3 = (data8 & 8) != 0;
	reg.bit.unused_bit5 = (data8 & 2) != 0;
	reg.w.HL = reg.w.HL + ( inc ? 1 : -1 );
	reg.w.BC -= 1;
	reg.bit.c = temp;
	reg.bit.pv = reg.w.BC != 0;
	sync(5);
	if (!repeat) return;
	if ( (reg.w.BC != 0) && !reg.bit.z ) {
		reg.w.PC -= 2;
		sync(5);
	}
}

template<bool inc, bool repeat> void Core_z80::op_ldx() {
	data8 = readByte(reg.w.HL, 3);
	writeByte(reg.w.DE, data8, 3);
	reg.w.DE +=  inc ? 1 : -1;
	reg.w.HL +=  inc ? 1 : -1;
	reg.w.BC -= 1;
	reg.bit.h = reg.bit.n = 0;
	reg.bit.pv = reg.w.BC != 0;
	data8 = reg.b.A + data8;
	reg.bit.unused_bit3 = (data8 & 8) != 0;
	reg.bit.unused_bit5 = (data8 & 2) != 0;
	sync(2);

	if (!repeat) return;
	if ( reg.w.BC != 0 ) {
		reg.w.PC -= 2;
		sync(5);
	}
}

template<bool al, bool addr, u8 regRefHi, u8 regRefLo> void Core_z80::op_ex() {
	u8 dataL, dataH;
	if (addr) {
		dataL = readByte(reg.w.SP);
		dataH = readByte(reg.w.SP + 1, 4);
	}

	if (addr) {
		exchange(&dataL, regPTR[regRefLo]);
		exchange(&dataH, regPTR[regRefHi]);
	} else if (!al) {
		exchange(&reg.b.D, &reg.b.H);
		exchange(&reg.b.E, &reg.b.L);
	} else if (regRefLo == _F) {
		exchange(&reg.b.A, &reg_al.b.A);
		exchange(&reg.b.F, &reg_al.b.F);
	} else {
		exchange(&reg.b.C, &reg_al.b.C);
		exchange(&reg.b.B, &reg_al.b.B);
		exchange(&reg.b.L, &reg_al.b.L);
		exchange(&reg.b.H, &reg_al.b.H);
		exchange(&reg.b.E, &reg_al.b.E);
		exchange(&reg.b.D, &reg_al.b.D);
	}

	if (addr) {
		writeByte(reg.w.SP, dataL);
		sync(2);
		writeByte(reg.w.SP + 1, dataH);
	}
}


template<bool inc, bool _8bit, bool addr, u8 regRef> void Core_z80::op_dec_inc() {
	displacement = 0;
	
	if (addr) {
		if (regRef == _IX || regRef == _IY) {
			displacement = readPc(3);
			sync(1);
		}
		sync(1);
		data8 = readByte(*regPTR16[regRef] + i8(displacement), 3);
	}

	if (addr && (regRef == _IX || regRef == _IY)) sync(4);
	
	if (_8bit) {
		u8 result = doIncDec<inc>(addr ? data8 : *regPTR[regRef]);
		
		if (!addr && (regRef == _IXH || regRef == _IYH || regRef == _IXL || regRef == _IYL) ) {
			sync(1);
		}

		if (addr) writeByte(*regPTR16[regRef] + i8(displacement), result, 3);
		else *regPTR[regRef] = result;
	} else {
		sync(2);
		*regPTR16[regRef] = *regPTR16[regRef] + (inc ? 1 : -1);
	}
}

template<u8 destRef, u8 srcRef, bool _8bit, bool addr, u16 (Core_z80::*op_l)(u16 src, u16 dest)>
void Core_z80::op_arithmetic() {
	if (_8bit) {
		if (srcRef == _DATA) data8 = readPc(3);
		else if (addr) {
			displacement = 0;
			if (srcRef == _IX || srcRef == _IY) {
				displacement = readPc(3);
				sync(5);
			}
			data8 = readByte(*regPTR16[srcRef] + i8(displacement), 3);
		}

		if (srcRef == _IXH || srcRef == _IYH || srcRef == _IXL || srcRef == _IYL
		|| srcRef == _IXH || srcRef == _IYH || srcRef == _IXL || srcRef == _IYL) {
			sync(1);
		}

		u8 temp = (this->*op_l)(reg.b.A, srcRef == _DATA || addr ? data8 : *regPTR[srcRef]);
		if (destRef != _NONE ) reg.b.A = temp;
		else setUndocumentedFlags( srcRef == _DATA || addr ? data8 : *regPTR[srcRef] ); //cp special
	} else {
		*regPTR16[destRef] = (this->*op_l)(*regPTR16[destRef], *regPTR16[srcRef]);
		sync(7);
	}
}

template<u8 mode> void Core_z80::op_im() {
	intr.imode = mode;
}

void Core_z80::op_halt() {
	stop = true;
	while(stop) {
		handle_interrupts();
		if (!stop) break;
		reg_r = (reg_r & 0x80) | (reg_r + 1) & 0x7F;
		sync(4);
	}
	intr.delay = true;
}

void Core_z80::op_nop() { }

void Core_z80::op_ccf() {
	reg.bit.h = reg.bit.c;
	reg.bit.n = 0;
	setUndocumentedFlags(reg.b.A);
	reg.bit.c = !reg.bit.c;
}

void Core_z80::op_scf() {
	reg.bit.c = 1;
	reg.bit.h = reg.bit.n = 0;
	setUndocumentedFlags(reg.b.A);
}

void Core_z80::op_neg() {
	reg.b.A = doSub8(0, reg.b.A);
}

void Core_z80::op_cpl() {
	reg.b.A = ~reg.b.A;
	reg.bit.h = reg.bit.n = 1;
	setUndocumentedFlags(reg.b.A);
}

void Core_z80::op_daa() {
	u8 correction = 0;
	u8 temp = reg.b.A;

	if (reg.bit.c || (reg.b.A > 0x99) ) {
		reg.bit.c = 1;
		correction = 6 << 4;
	} else reg.bit.c = 0;

	if (reg.bit.h || ( (reg.b.A & 0x0F) > 0x09) ) correction |= 6;

	if (!reg.bit.n) {
		reg.b.A += correction;
	} else {
		reg.b.A -= correction;
	}
	
	reg.bit.h = (temp ^ reg.b.A) & 0x10;
	setUndocumentedFlags(reg.b.A);
	setStandardFalgs(reg.b.A);
}

template<u8 regRef, bool addr, u8 mode, u8 adjust> void Core_z80::op_bitM() {
	if (addr) {
		data8 = readByte( *regPTR16[regRef], 4);
	} else {
		data8 = *regPTR[regRef];
	}
	switch ( mode ) {
		case _SET: data8 = doSet(adjust, data8); break;
		case _RESET: data8 = doRes(adjust, data8); break;
		case _RL: data8 = doRL<false, adjust> (data8); break;
		case _RLC: data8 = doRL<true, adjust> (data8); break;
		case _RR: data8 = doRR<false, adjust> (data8); break;
		case _RRC: data8 = doRR<true, adjust> (data8); break;
		case _SLL: data8 = doSL<false> (data8); break;
		case _SLA: data8 = doSL<true> (data8); break;
		case _SRL: data8 = doSR<false> (data8); break;
		case _SRA: data8 = doSR<true> (data8); break;
	}
	if (addr) {
		writeByte(*regPTR16[regRef], data8);
	} else {
		*regPTR[regRef] = data8;
	}
}

void Core_z80::op_rld() {
	data8 = readByte(reg.w.HL);
	writeByte(reg.w.HL, data8 << 4 | (data8 & 0xF), 4);
	u8 nb = reg.b.A;
	reg.b.A = (data8 >> 4) | (reg.b.A & 0xF0);
	writeByte(reg.w.HL, data8 << 4 | (nb & 0xF));
	setStandardFalgs(reg.b.A);
	setUndocumentedFlags(reg.b.A);
	reg.bit.n = reg.bit.h = 0;
}

void Core_z80::op_rrd() {
	data8 = readByte(reg.w.HL);
	u8 nb = reg.b.A;
	reg.b.A = (data8 & 0xF) | (reg.b.A & 0xF0);
	writeByte(reg.w.HL, nb << 4 | (data8 & 0xF), 4);
	writeByte(reg.w.HL, nb << 4 | (data8 >> 4));
	setStandardFalgs(reg.b.A);
	setUndocumentedFlags(reg.b.A);
	reg.bit.n = reg.bit.h = 0;
}

template<u8 Ref, bool addr, u8 bit> void Core_z80::op_bit() {
	if (Ref != _IX && Ref != _IY) displacement = 0;
	if (addr) {
		data16 = *regPTR16[Ref] + i8(displacement);
		data8 = readByte( data16 , 4);
	}
	data8 = (addr ? data8 : *regPTR[Ref]) & (1 << bit);
	reg.b.F &= FLAG_C;
	reg.bit.h = 1;
	if ( data8 == 0) {
		reg.bit.pv = reg.bit.z = 1;
	} else {
		if (bit == 7) reg.bit.s = 1;
	}

	if (!addr) {
		setUndocumentedFlags( *regPTR[Ref] );
	} else {
		if (Ref == _HL) setUndocumentedFlags( reg.b.H );
		else setUndocumentedFlags( (data16 >> 8) & 0xFF );
	}
}

template<i8 cond, u8 regRef> void Core_z80::op_jp() {
	if (regRef == _DATA) {
		data16 = readPc(3);
		data16 |= readPc(3) << 8;
	}

	if (cond == -1 || getCC(cond)) {
		reg.w.PC = regRef == _DATA ? data16 : *regPTR16[regRef];
	}
}

template<i8 cond> void Core_z80::op_jr() {
	displacement = readPc(3);
	if (cond == -1 || getCC(cond)) {
		reg.w.PC += i8(displacement);
		sync(5);
	}
}

void Core_z80::op_djnz() {
	sync(1);
	displacement = readPc(3);
	if (--reg.b.B) {
		reg.w.PC += i8(displacement);
		sync(5);
	}
}

template<i8 cond> void Core_z80::op_call() {
	data16 = readPc(3);
	data16 |= readPc(3) << 8;
	if (cond == -1 || getCC(cond)) {
		sync(1);
		writeStack(reg.w.PC);
		reg.w.PC = data16;
	}
}

template<i8 cond> void Core_z80::op_ret() {
	if (cond != -1) sync(1);
	if (cond == -1 || getCC(cond)) {
		reg.w.PC = readStack();
	}
}

void Core_z80::op_reti() {
	reg.w.PC = readStack();
	intr.iff1 = intr.iff2;
}

void Core_z80::op_retn() {
	reg.w.PC = readStack();
	intr.iff1 = intr.iff2;
}

template<u8 begin> void Core_z80::op_rst() {
	sync(1);
	writeStack(reg.w.PC);
	reg.b.PCh = 0;
	reg.b.PCl = begin;
}

template<u8 regRef> void Core_z80::op_out() {
	if (regRef == _DATA) {
		data8 = readPc(3);
		writeIo(data8, reg.b.A, 4);
	} else if (regRef == _NONE) {
		writeIo(reg.b.C, 0, 4);
	} else {
		writeIo(reg.b.C, *regPTR[regRef], 4);
	}
}

template<u8 regRef> void Core_z80::op_in() {
	if (regRef == _DATA) {
		data8 = readPc(3);
		reg.b.A = readIo(data8, 4);
	} else {
		data8 = readIo(reg.b.C, 4);
		setStandardFalgs(data8);
		setUndocumentedFlags(data8);
		reg.bit.h = reg.bit.n = 0;
		if(regRef != _F) *regPTR[regRef] = data8;
	}
}

template<bool repeat, bool inc> void Core_z80::op_outx() {
	sync(1);
	data8 = readByte(reg.w.HL);
	writeIo(reg.b.C, data8, 4);
	reg.b.B--;
	reg.w.HL += inc ? 1 : -1;

	calcInOutIncDecFlags<inc, false>();

	if (!repeat) return;
	if ( reg.b.B ) {
		reg.w.PC -= 2;
		sync(5);
	}
}

template<bool repeat, bool inc> void Core_z80::op_inx() {
	sync(1);
	data8 = readIo(reg.b.C, 3);
	writeByte(reg.w.HL, data8, 4);
	reg.b.B--;
	reg.w.HL +=  inc ? 1 : -1;

	calcInOutIncDecFlags<inc, true>();

	if (!repeat) return;
	if ( reg.b.B ) {
		reg.w.PC -= 2;
		sync(5);
	}
}
